





alSynSetFXParam(3P)                                        alSynSetFXParam(3P)







NAME

     alSynSetFXParam - set an effect parameter to the specified value.





SYNOPSIS

     #include <libaudio.h>



     void alSynSetFXParam(ALSynth *drvr, ALFxRef fx, s16 paramID, void *param);





PARAMETERS

     drvr      pointer to the synthsizer driver.



     fx        pointer to an effect.



     paramID   id for the parameter to be modified.



     param     pointer to the new parameter value.



DESCRIPTION

     Set the specified effect parameter to the value pointed to by *param. The

     paramID is an index into the array of s32 values, params, passed in the

     ALSynConfig struct when alInit is called. The first two values, which

     specify the number of delay sections, and the size of the fx delay line

     respectively, can not be changed. The remaining values are divided into

     sections, that consist of eight parameters each. The parameters of each

     section are:



     input The location in the delay line that the feedforward multiplier uses

     as its input. It is also where the feedback multiplier's result is

     stored.  param is specified in samples, and must be 8 sample aligned. The

     value pointed to by param should be an unsigned 32 bit integer. The input

     parameter parameter has an offset of 0 from the start of the delay

     section.



     output The location in the delay line that the feedback multiplier uses

     as its input. It is also where the feedforward mutltiplier stores it's

     result if there is no output resampler.  output is specified in samples,

     and must be 8 sample aligned. The value pointed to by param should be an

     unsigned 32 bit integer. The output parameter has and offset of 1 from

     the start of the delay section.



     fbcoef The delay section's feed back coefficient. The value pointed to by

     param should be a signed 32 bit value. Only the lower 15 bits are

     significant, and the upper bits are sign extended. The lower 15 bits can

     be thought of as a fraction representing values from 1 to -1. The fbcoef

     parameter has an offset of 2 from the start of the delay section.



     ffcoef The delay section's feed foward coefficient. The value pointed to

     by param should be a signed 32 bit value. Only the lower 15 bits are

     significant, and the upper bits are sign extended. The lower 15 bits can

     be thought of as a fraction representing values from 1 to -1. The ffcoef







                                                                        Page 1













alSynSetFXParam(3P)                                        alSynSetFXParam(3P)







     parameter has an offset of 3 from the start of the delay section.



     gain The delay section's gain value, i.e., it's contribution to the final

     effect output. The value pointed to by param should be a signed 32 bit

     value. Only the lower 15 bits are significant, and the upper bits are

     sign extended. The lower 15 bits can be thought of as a fraction

     representing values from 1 to -1. The gain parameter has an offset of 4

     from the start of the delay section.



     chorus rate The delay section's chorus rate parameter specifies the

     modulation frequency of the output tap position of the delay line, i.e.,

     how quickly the tap position will be modulated. The value of this

     parameter is (frequency/sample rate)*2^25.  For example, a modulation

     frequency of .5Hz at a synthesizer sample rate of 44.1kHz would be

     (.5/44100)*33,554,432 = 380. The chorus rate parameter has an offset of 5

     from the start of the delay section.



     chorus depth The delay section's chorus depth parameter specifies the

     modulation depth, or pitch change, of the effect. The parameter is

     specified approximately in hundredths of a cent. So a modulation depth of

     +/-25 cents, or a quarter of a semitone, would be 2500. The approximation

     to cents is good over the range useful for musical chorusing and

     flanging, i.e., less than a few semitones. The error at 1 semitone (100

     cents) is about 3 cents and at 3 semitones is about 30 cents.  The chorus

     depth parameter has an offset of 5 from the start of the delay section.



     lpfilt The delay section's low pass filter coefficient. The value pointed

     to by param should be a signed 32 bit value. Only the lower 15 bits are

     significant, and the upper bits are sign extended. The lower 15 bits can

     be thought of as a fraction representing values from 1 to -1. The lpfilt

     parameter has an offset of 7 from the start of the delay section.



EXAMPLE

     The following code example shows how to set the ffcoef of the third delay

     section, to a value of .5



            ALFxRef   fxref;

            s16       paramID;

            s32       section = 2; /* thirdd section */

            s32       offset = 3;  /* ffcoef has an offset of 3 */

            s32       value = 0x4000; /* .5 expressed as a */

                                      /* 15 bit fraction */



            fxref = alSynGetFXRef(synth, 0, 0);

            paramID = (section * 8) + param_number + 2;

            alSynSetFXParam(synth,fxref,paramID,&value);





WARNING

     alSynSetFXParam does very minimal checking on the values passed to it.

     You can easily create a feedback loop that at best will be unpleasant.

     (At worst painful!) Also, alSynSetFXParam does not check to verify that







                                                                        Page 2













alSynSetFXParam(3P)                                        alSynSetFXParam(3P)







     the paramID is valid. Sending a param with an invalid paramID can cause

     unpredictable results.







SEE ALSO

     alSynAllocFX(3P), alSynFreeFX(3P), alSynGetFXRef(3P)

































































































                                                                        Page 3







