





gSPModifyVertex(3P)                                        gSPModifyVertex(3P)







NAME

     gSPModifyVertex, gsSPModifyVertex - Modify a vertex after it is loaded

     into the RSP.



C SPECIFICATION

     #include "gbi.h"



     gSPModifyVertex(Gfx *gdl, unsigned int vtx, unsigned int where, int val)



     gsSPModifyVertex(unsigned int vtx, unsigned int where, int val)



     gsSPNearClip(nc)





PARAMETERS

     *gdl      graphics display list pointer.



     vtx       Which of the RSP's vertices (0-15) to modify.



     where     What part of the vertex to modify.  One of:

                  G_MWO_POINT_RGBA

                  G_MWO_POINT_ST

                  G_MWO_POINT_XYSCREEN

                  G_MWO_POINT_ZSCREEN





     val       New value for that part of the vertex (a 32 bit integer

               number).





DESCRIPTION

     This is an advanced command.  A good understanding of how vertices work

     in the RSP microcode is suggested before using this command (see

     gSPVertex (3P)).



     This command allows you to modify certain sections of a vertex after it

     has been sent to the RSP (with the gSPVertex (3P) command).  This is

     useful for vertices which are shared between 2 or more triangles, but

     which must have different properties when associated with one triangle

     than when associated with a different triangle.  For example two adjacent

     triangles which both need smooth shaded color, but one is smooth shaded

     red-to-yellow and the other is smooth shaded green-to-cyan.  In this case

     the vertex which is shared by both triangles is sent (using the gSPVertex

     (3P)) command with red/yellow color; the first triangle is drawn; the

     gSPModifyVertex (3P) command is used to change the color to green/cyan;

     and the second triangle is drawn.  The primary use of the gSPModifyVertex

     (3P) command is for modifying the texture coordinate of a vertex so that

     a vertex which is shared by 2 triangles with different textures and

     different texture coordinate spaces can contain the texture coordinate

     for the first texture, and then be modified to contain the texture

     coordinate for the second texture.









                                                                        Page 1













gSPModifyVertex(3P)                                        gSPModifyVertex(3P)







     The gSPModifyVertex (3P) command is faster than sending a new Vertex

     command with a different but similar vertex because no transformations or

     lighting are done to the vertex when the gSPModifyVertex (3P) command is

     used.



     The where parameter specifies which part of the vertex is to be modified:

        G_MWO_POINT_RGBA

            change the color of the vertex.  The val parameter is interpreted

            as 4 bytes: red (high byte), green, blue, and alpha (low byte).

        G_MWO_POINT_ST

            change the S and T values (texture coordinates of the vertex).

            The high 16 bits of val is the S coordinate, and the low 16 bits

            are the T coordinate.  Each coordinate is an S10.5 number.

        G_MWO_POINT_XYSCREEN

            change the screen coordinates of the vertex.  The high 16 bits of

            val are the X coordinate and the low 16 bits are the Y coordinate.

            Both coordinates are S13.2 numbers with 0,0 being the upper left

            of the screen, positive X going right, and positive Y going down.

        G_MWO_POINT_ZSCREEN

            change the screen Z coordinate of the vertex.  The entire 32 bit

            val is taken as the new screen Z value.  It is a 16.16 number in

            the range 0x00000000 to 0x03ff0000.





NOTE

     Lighting is not performed after a gSPModifyVertex (3P) command, so

     modifying the color of the vertex with G_MWO_POINT_RGBA is just that -

     modifying the actual color which will be output, and not a modification

     of normal values. This means it cannot be used to update vertex normals

     for lighting.



     The S and T coordinates supplied in the gSPModifyVertex (3P) command are

     never multiplied by the texture scale (from the gSPTexture (3P) command),

     so they must be prescaled by you before being sent.  For example if you

     want to use a texture scale of 1/2 (0x8000) then the S and T values you

     send with the gSPModifyVertex (3P) command should be half the value of

     the equivalent values used with the gSPVertex (3P) command.





EXAMPLE

     To share a vertex between 2 triangles with different textures and texture

     coordinates on the 2 triangles:



          /* load the vertices with gSPVertex */

          gSPVertex(...);



          /* load the texture for triangle 1 */

          gDPLoadTextureBlock(...);



          /* draw triangle 1 which uses vertex #3 */











                                                                        Page 2













gSPModifyVertex(3P)                                        gSPModifyVertex(3P)







          gSP1Triangle(glistp++, 1,2,3,0);



          /* make vertex #3 have values S=3.0  T=2.5 */

          gSPModifyVertex(glistp++, 3, G_MWO_POINT_ST, 0x00600050);



          /* load the texture for triangle 2 */

          gDPLoadTextureBlock(...);



          /* draw triangle 2 which also uses vertex #3 */

          gSP1Triangle(glistp++, 1,2,3,0);





SEE ALSO

     gSPVertex(3P), gSPTexture(3P), gSP1Triangle(3P),



















































































                                                                        Page 3







