





gspSprite2D(3P)                                                gspSprite2D(3P)







NAME

     gspSprite2D, gspSprite2D.dram - high-quality 2D sprite geometry RSP

     microcode.





DESCRIPTION

     This is the optimized, full-featured 2D sprite geometry RSP microcode.

     It supports automatic subdivision and loading of arbitrarily sized dram

     resident images of all supported texture format sizes and types into

     gSPTextureRectangle(3P) commands sent directly to the RDP. Additionally,

     images can be optionally scaled up or mirrored in the X or Y directions.





USAGE

     The sprite microcode is accessed through a combination of 2 new commands.

     The first command, guSprite2DInit(3P) is used to initialize a sprite

     structure with user specified values which are then passed to the

     gSPSprite2D(3P) command for actual processing by the microcode



     A sample code fragment for specifying a sprite would be:



     #include "gu.h"

     #include "gbi.h"



     uSprite MySprite;



     guSprite2DInit(&MySprite, ImagePointer, TlutPointer,

                    ImageWidth, RectangleWidth, RectangleHeight,

                    ImageType, ImageSize,

                    TextureScaleX, TextureScaleY,

                    FlipTextureX, FlipTextureY,

                    TextureStartS, TextureStartT,

                    TranslateHorizontal, TranslateVertical);



     gSPSprite2D(glistp++, OS_K0_TO_PHYSICAL(&MySprite));









     These parameters are explained below.







C SPECIFICATION

     typedef struct {

       void *SourceImagePointer;

       void *TlutPointer;

       short Stride;

       short SubImageWidth;

       short SubImageHeight;

       char  SourceImageType;

       char  SourceImageBitSize;







                                                                        Page 1













gspSprite2D(3P)                                                gspSprite2D(3P)







       short ScaleX;

       short ScaleY;

       char  FlipTextureX;

       char  FlipTextureY;

       short SourceImageOffsetS;

       short SourceImageOffsetT;

       short PScreenX;

       short PScreenY;

       /* 30 bytes for above */



       /* padding to bring structure size to 64 bit allignment */

       char dummy[2];



     } uSprite_t;



     typedef union {

       uSprite_t  s;



       /* Need to make sure this is 64 bit aligned */

       long long int         force_structure_allignment[4];

     } uSprite;





     void guSprite2DInit(uSprite *SpritePointer,

                   void *SourceImagePointer,

                   void *TlutPointer,

                   short Stride,

                   short SubImageWidth,

                   short SubImageHeight,

                   char  SourceImageType,

                   char  SourceImageBitSize,

                   short ScaleX,

                   short ScaleY,

                   char  FlipTextureX,

                   char  FlipTextureY,

                   short SourceImageOffsetS,

                   short SourceImageOffsetT,

                   short PScreenX,

                   short PScreenY);









PARAMETERS

     SpritePointer The address of the sprite structure into which the

     parameters will be written.



     SourceImagePointer The address of the base of the texture image in memory

     out of which a subrectangle is to be displayed.



     TlutPointer The address of an optional color index table for used with CI

     images.  Use Null for non-CI images.







                                                                        Page 2













gspSprite2D(3P)                                                gspSprite2D(3P)







     Stride The width in texels of the original base image in memory.



     SubImageWidth The width in texels of the subimage which is to be

     displayed.



     SubImageHeight The height in texels of the subimage which is to be

     displayed.



     SourceImageType The format of the texture image in memory. All supported

     hardware texture formats are allowed, such as

      G_IM_FMT_RGBA  or  G_IM_FMT_CI.



     SourceImageBitSize The number of bits per texel of the input image. All

     supported hardware texture sizes are allowed, such as

      G_IM_SIZ_32b  or  G_IM_SIZ_4b.



     ScaleX The s5.10 fixed point X axis texture to pixel scaling ratio which

     is to be applied to the input image. A value of 1024 specifies 1 to 1

     scaling. A value of 512 specifies that each input texel should be scaled

     up to 2 output screen pixels. Scale values should be <= 1024 in order to

     prevent sampling artifacts from occuring. Scale values must be positive,

     use the FlipTextureX parameter to create negatively scaled images.



     ScaleY The s5.10 fixed point Y axis texture to pixel scaling ratio which

     is to be applied to the input image. A value of 1024 specifies 1 to 1

     scaling. A value of 512 specifies that each input row should be scaled up

     to 2 output screen rows. Scale values should be <= 1024 in order to

     prevent sampling artifacts from occuring. Scale values must be positive,

     use the FlipTextureY parameter to create negatively scaled images.



     FlipTextureX Specifies whether the image should be mirrored in the X

     direction before display



     FlipTextureY Specifies whether the image should be mirrored in the Y

     direction before display



     SourceImageOffsetS The offset in texel columns from the origin of the

     input base image where the texture subrectangle which is to be displayed

     starts.



     SourceImageOffsetT The offset in texel rows from the origin of the input

     base image where the texture subrectangle which is to be displayed

     starts.



     PScreenX Specifies the starting X location in screen coordinates of the

     output image.  The origin is in the upper left corner of the screen.



     PScreenY Specifies the starting Y location in screen coordinates of the

     output image.  The origin is in the upper left corner of the screen.













                                                                        Page 3













gspSprite2D(3P)                                                gspSprite2D(3P)







GBI

     The following GBI commands are not supported with this microcode:

          gSPLine3D gSP1Triangle





NOTE

     The sprite microcode has no direct support for ZBuffering. This is

     unecessary as ZBuffering can be accomplished outside of the sprite

     microcode by setting up the proper rendering mode and making use of the

     hardware primitive depth register. A code fragment for doing ZBuffered

     sprites is:







     gDPSetRenderMode(glistp++, G_RM_AA_ZB_OPA_SURF, G_RM_AA_ZB_OPA_SURF2);

     gDPSetDepthSource(glistp++, G_ZS_PRIM);

     gDPSetCombineMode(glistp++, G_CC_DECALRGB, G_CC_DECALRGB);

     gDPSetPrimDepth(glistp++, ZBufferValue, 0);



     guSprite2DInit(&MySprite, ImagePointer, TlutPointer,

                    ImageWidth, RectangleWidth, RectangleHeight,

                    ImageType, ImageSize, TextureScaleX, TextureScaleY,

                    FlipTextureX, FlipTextureY, TextureStartS, TextureStartT,

                    TranslateHorizontal, TranslateVertical);



     gSPSprite2D(glistp++, OS_K0_TO_PHYSICAL(&MySprite));















WARNINGS, LIMITATIONS, BUGS, WORKAROUNDS

     Images which have been non-unit scaled and flipped in the Y axis will not

     translate vertically by sub-pixel amounts smoothly, they will exhibit a

     certain amount of jumping. The solution is to translate non-unit scaled Y

     flipped images by unit amounts in the vertical direction.



     The Sprite Microcode was designed to handle images which have been

     optionally scaled up. Images can also be scaled down with some attendant

     artifacts. Beware that the TextureScaleX and TextureScaleY parameters are

     s5.10, however they are restricted to being positive, thus the largest

     usuable scale value is 32767, which corresponds to a texel to pixel ratio

     of 31.999.



     Texture images which are either scaled in the Y axis or placed on a

     subpixel scanline boundary require filtering by the hardware texture

     filter unit. This filtering requires that at least 1 extra row of the

     input image be loaded into the texture memory so that the filtering can

     occur. Since the texture memory is limited to 4KB, this means that the

     following restrictions are in effect:







                                                                        Page 4













gspSprite2D(3P)                                                gspSprite2D(3P)







         32 bit subrectangles which are scaled in the Y direction are clamped

     by the microcode to being a maximum of 512 texels wide.



     additionally,



          32 bit subrectangles which are placed starting on a subpixel

     boundary in the Y direction and are larger than 512 texels in width are

     clamped by the microcode to being on an integral scan line boundary.



     Note that this filtering requirement also means that when displaying a Y

     scaled image, you will need to offset your starting T value by 1 to

     prevent artifacts resulting from filtering with out of bound texels.



     Y flipped Images which have been scaled vertically by non power of 2

     amounts will be slightly different heights than their non-flipped

     versions. If doing an animation involving scaled flipped and non-flipped

     textures, restrict the Y scale value to being a power of 2.





SEE ALSO

     gspLine3D(3P), gspTurbo3D(3P) gspFast3D(3P) osSpTaskStart(3P)





































































                                                                        Page 5







