





osContInit(3P)                                                  osContInit(3P)







NAME

     osContInit - poll for and reset game controllers



SYNOPSIS

     #include <ultra64.h>



     s32 osContInit(OSMesgQueue *mq, u8 *bitpattern, OSContStatus *status);





DESCRIPTION

     The osContInit call resets all the game controllers and returns a bit

     pattern to indicate which game controllers are connected.  It also

     returns game controller's type and status back to status. The user must

     supply a block of memory large enough for MAXCONTROLLERS structures of

     type OSContStatus. The message queue mq must be an initialized message

     queue associated with the OS_EVENT_SI event.  See osSetEventMesg(3P) for

     details on how to create this association.  Since the osContInit routine

     will wait on the message queue mq, this queue must not be shared.  A

     value of 0 is returned if the calls succeed, otherwise -1 is returned.



     The game controller's type can be one of the following defined constants:



          CONT_ABSOLUTE  The controller contains counters inside and

                         sends the joystick data as an absolute value.

          CONT_RELATIVE  The controller contains counters inside and

                         sends the joystick data as a relative value.

          CONT_JOYPORT   The controller has a joyport which connects

                         to a exchangeable CONTROLLER PAK.



     The game controller's status is used for a CONTROLLER PAK and can be one

     of the following defined constants:



          CONT_CARD_ON        This bit is set if CONTROLLER PAK is

                              connected to the controller.

          CONT_CARD_PULL      After controller is reset, if the CONTROLLER

                              PAK is pulled out, this bit will be set.

          CONT_ADDR_CRC_ER    This bit is set if the address is not

                              transfered to the joyport correctly.



     The error returned from controllers can be:



          CONT_NO_RESPONSE_ERROR   The controller doesn't respond.

          CONT_OVERRUN_ERROR       The controller sends data at higher

                                   data transfer rate than the hardware

                                   handling capability.



     The user should ignore the data when detects CONT_OVERRUN_ERROR.



EXAMPLE

     main()

     {

          OSMesgQueue      intMesgQueue;







                                                                        Page 1













osContInit(3P)                                                  osContInit(3P)







         OSMesg           intMesgBuf[1];

          OSContStatus     sdata[MAXCONTROLLERS];



          osCreateMesgQueue(&intMesgQueue, intMesgBuf, 1);

          osSetEventMesg(OS_EVENT_SI, &intMesgQueue, dummyMessage);

          osContInit(&intMesgQueue, &pattern, &sdata[0]);

     }



SEE ALSO

     osContReset(3P), osContStartQuery(3P), osContGetQuery(3P),

     osContStartReadData(3P), osContGetReadData(3P), osContSetCh(3P)

























































































                                                                        Page 2







