





osContStartQuery(3P)                                      osContStartQuery(3P)







NAME

     osContStartQuery, osContGetQuery - obtain game controller status and type



SYNOPSIS

     #include <ultra64.h>



     s32 osContStartQuery(OSMesgQueue *mq);



     void osContGetQuery(OSContStatus *status);





DESCRIPTION

     The osContStartQuery call issues a query command to game controllers to

     obtain game controller status and type, and the osContGetQuery call

     returns game controller's type and status back to status. The user must

     supply a block of memory large enough for MAXCONTROLLERS structures of

     type OSContStatus. The message queue mq must be an initialized message

     queue associated with the OS_EVENT_SI event.  See osSetEventMesg(3P) for

     details on how to create this association.  The query command will take

     around 2 milliseconds to pull data back from game controllers.  Thus, the

     osRecvMesg call on the message queue mq can be used to wait for this

     event.



     The game controller's type can be one of the following defined constants:



          CONT_ABSOLUTE  The controller contains counters inside and

                         sends the joystick data as an absolute value.

          CONT_RELATIVE  The controller contains counters inside and

                         sends the joystick data as a relative value.

          CONT_JOYPORT   The controller has a joyport which connects

                         to an exchangeable CONTROLLER PAK.



     The game controller's status are used for CONTROLLER PAK and can be one

     of the following defined constants:



          CONT_CARD_ON      This bit is set if CONTROLLER PAK is connected

                            to the controller.

          CONT_CARD_PULL    After controller is reset, if the CONTROLLER

                            PAK is pulled out, this bit will be set.

          CONT_ADDR_CRC_ER  This bit is set if the address is not

                            transfered to the joyport correctly.



     The error number return from game controllers can be:



          CONT_NO_RESPONSE_ERROR The controller doesn't respond.

          CONT_OVERRUN_ERROR     The controller sends data at higher

                                 data transfer rate than the hardware

                                 handling capability.

     The user should ignore the data when detects CONT_OVERRUN_ERROR.













                                                                        Page 1













osContStartQuery(3P)                                      osContStartQuery(3P)







EXAMPLE

     main()

     {

          OSMesgQueue      intMesgQueue;

          OSMesg          intMesgBuf[NUM_MESSAGE];

          OSContStatus     sdata[MAXCONTROLLERS];



          osCreateMesgQueue(&intMesgQueue, intMesgBuf, NUM_MESSAGE);

          osSetEventMesg(OS_EVENT_SI, &intMesgQueue, dummyMessage);

          osContInit(&intMesgQueue, &pattern, &sdata[0]);

          osContStartQuery(&intMesgQueue);

                         .

                         .

          osRecvMesg(&intMesgQueue, &actualMesg, OS_MESG_BLOCK);

          osContGetQuery(&sdata[0]);

     }



SEE ALSO

     osContInit (3P), osContReset (3P), osContStartReadData (3P),

     osContGetReadData (3P), osContSetCh (3P)







































































                                                                        Page 2







