





osContStartReadData(3P)                                osContStartReadData(3P)







NAME

     osContStartReadData, osContGetReadData - obtain game controller input

     settings



SYNOPSIS

     #include <ultra64.h>



     s32 osContStartReadData(OSMesgQueue *mq);



     void osContGetReadData(OSContPad *pad);





DESCRIPTION

     The osContStartReadData call issues a read data command to obtain game

     controller input settings, and the osContGetReadData call returns 3D

     Stick data and button settings to pad. The user must supply a block of

     memory large enough for MAXCONTROLLERS structures of type OSContStatus.

     The message queue mq must be an initialized message queue associated with

     the OS_EVENT_SI event.  See osSetEventMesg(3P) for details on how to

     create this association.  The read data command will take around 2

     milliseconds to pull data back from game controllers.  Thus, the

     osRecvMesg call on the message queue mq can be used to wait for this

     event.



     The 3D Stick data is of type signed char and in the range between 80 and

     -80. The game controller's input settings can be the following defined

     constants:



          START_BUTTON   Start Button

          A_BUTTON       A Button

          B_BUTTON       B Button

          U_CBUTTONS     C Button Unit Up

          D_CBUTTONS     C Button Unit Down

          L_CBUTTONS     C Button Unit Left

          R_CBUTTONS     C Button Unit Right

          U_JPAD         Joy Pad Up

          D_JPAD         Joy Pad Down

          L_JPAD         Joy Pad Left

          R_JPAD         Joy Pad Right

          Z_TRIG         Z Trigger

          L_TRIG         L Trigger

          R_TRIG         R Trigger



     The error number return from game controllers can be:



          CONT_NO_RESPONSE_ERROR The controller doesn't respond.

          CONT_OVERRUN_ERROR     The controller sends data at higher

                                 data transfer rate than the hardware

                                 handling capability.













                                                                        Page 1













osContStartReadData(3P)                                osContStartReadData(3P)







     The user should ignore the data when detects CONT_OVERRUN_ERROR.





EXAMPLE

     main()

     {

          OSMesgQueue      intMesgQueue;

          OSMesg          intMesgBuf[NUM_MESSAGE];

          OSContStatus     sdata[MAXCONTROLLERS];

          OSContPad        rdata[MAXCONTROLLERS];



          osCreateMesgQueue(&intMesgQueue, intMesgBuf, NUM_MESSAGE);

          osSetEventMesg(OS_EVENT_SI, &intMesgQueue, dummyMessage);

          osContInit(&intMesgQueue, &pattern, &sdata[0]);

          osContStartReadData(&intMesgQueue);

                         .

                         .

          osRecvMesg(&intMesgQueue, &actualMesg, OS_MESG_BLOCK);

          osContGetReadData(&rdata[0]);

     }



SEE ALSO

     osContInit (3P), osContReset (3P), osContStartQuery (3P), osContSetCh

     (3P)































































                                                                        Page 2







