





osProfile(3P)                                                    osProfile(3P)







NAME

     osProfileInit, osProfileStart, osProfileStop, osProfileFlush - execution

     time profile for disjoint text spaces



SYNOPSIS

     #include <ultra64.h>



     void osProfileInit(OSProf *profp, u32 profcnt);



     void osProfileStart(u32 microseconds);



     void osProfileStop(void);



     void osProfileFlush(void);





DESCRIPTION

     Profiling provides CPU-use statistics by profiling the amount of CPU time

     expended by a program in disjoint text spaces.



     Profiling generates the statistics by creating an execution histogram for

     a current process.  The histogram is defined for a list of regions of

     program code to be profiled specified by members of the profp array:

     profp[0], profp[1], ..., profp[profcnt-1].  The host-side application

     gperf (1P) copies the histogram data to the host and prints a report

     detailing the relative time spent processing each function.



     The OSProf data structure has the following elements:





               u16 *histo_base

                         Pointer to an array of u16 counters.



               u32 histo_size

                         Number of counters in the histo_base buffer.

                         Profiling requires one counter for each 32 bit word

                         in the segment.  (one counter for each instruction)



               u32 *text_start

                         Pointer to first instruction in segment to be

                         profiled.



               u32 *text_end

                         Pointer to last instruction in segment to be

                         profiled.



     Each prof entry specifies a region of text space that needs to be

     profiled.  If an instruction falls outside the bounds specified by the

     profp array, an internal 32-bit overflow counter will be incremented.

     The histogram counters are always 16-bits.  There is no protection for

     overflow of these counters.









                                                                        Page 1













osProfile(3P)                                                    osProfile(3P)







     osProfileInit initializes the profiled segment count buffers and starts

     an I/O thread that communicates profile data to the host when requested

     by gperf (1P). The user passes a pointer to an array of OSProf

     structures, profp , each element of which describes a segment to be

     profiled.  The profcnt parameter indicates number of profiled segments.

     osProfileInit clears the counters for each profiled segment.  A thread is

     started in order to service requests for profile data from gperf (1P).

     Profiling data can be dumped to the host any time after osProfileInit is

     called.  osProfileInit should be called before any of the other profiler

     functions.



     The osProfileStart function is called to start the profiler's interval

     counter.  The microseconds parameter is used to set the period of this

     timer.  A typical value is 10000 microseconds (10 milliseconds).   The

     minimum timer period is defined by PROF_MIN_INTERVAL. Every time an

     interrupt is generated by the counter, the profiler determines which

     profiled segment the PC belongs to and increments the proper counter in

     the counter buffer for this segment.  If the PC does not lie within any

     profiled segment, an internal overflow register is incremented.  There is

     no protection for counter overflow.



     The osProfileStop function is called to turn off profiling.  Profiling

     data remains in memory but is not incremented further.  The thread

     started by osProfileInit to send profile data to the host continues to

     run.  may be called again to resume profiling.



     osProfileFlush can be used to programatically transfer profile data to

     the gperf (1P) program assuming the gperf program is running in server

     mode.  Gperf (1P) can also send requests to the profiler's I/O thread at

     any time while the program is executing (non-server mode).



     The following errors are reported when using the debug library:



     ERR_OSPROFILEINIT_STR

               Profile counter is running, call osProfileStop before init



     ERR_OSPROFILEINIT_CNT

               Profcnt is an illegal value



     ERR_OSPROFILEINIT_ALN

               Histo_base pointer must be 32-bit aligned



     ERR_OSPROFILEINIT_ORD

               Text_start >= text_end



     ERR_OSPROFILEINIT_SIZ

               Histo_size is an illegal size



     ERR_OSPROFILESTART_TIME

               Microseconds is < PROF_MIN_INTERVAL











                                                                        Page 2













osProfile(3P)                                                    osProfile(3P)







     ERR_OSPROFILESTART_FLAG

               Profiling has already been started



     ERR_OSPROFILESTOP_FLAG

               Profiling has already been stopped



     ERR_OSPROFILESTOP_TIMER

               No profile timer to stop



BUGS

     Calling osProfileFlush when gperf is not running can cause unexpected

     results later.



     Gperf (and thus profiling) does not work with programs whose functions

     have been rearranged in the executable objects for optimization.



     Currently, profiling only supports 16-bit counters.  It would be nice to

     also use 32-bit counters.



     Profiling is not available in the libultra_rom library.



     There is no check for overlapping text segments in the OSProf array.



SEE ALSO

     gperf(1P), makerom(1P)





























































                                                                        Page 3







