





uhOpenGame(3P)                                                  uhOpenGame(3P)







NAME

     uhOpenGame, uhCloseGame, uhReadGame, uhWriteGame, uhReadRamrom,

     uhWriteRamrom  - data transfer facilities from/to host and game



SYNOPSIS

     #include <ultrahost.h>



     int uhOpenGame(const char *device);



     int uhReadGame(int fd, void *addr, int nbytes);



     int uhWriteGame(int fd, void *addr, int nbytes);



     int uhReadRamrom(int fd, void *ramrom_addr, void *data, int nbytes);



     int uhWriteRamrom(int fd, void *ramrom_addr, void *data, int nbytes);



     int uhCloseGame(int fd);





DESCRIPTION

     These routines allow a host application to set up a communications

     channel for the transfer of raw data between the host and the game.  They

     work in concert with the routines described in osReadHost(3P) on the game

     side.



     The uhOpenGame routine opens the given device special file and configures

     it to listen for game I/O events.  Which device should be opened is

     dependent on which read and write calls will be needed. For uhReadGame

     and uhWriteGame, uhOpenGame should open the device /dev/u64_data. For

     uhReadRamrom and uhWriteRamrom, uhOpenGame should open /dev/u64. Note

     that this is a change from previous releases that specified /dev/u64 for

     all calls.  The file descriptor returned by uhOpenGame is passed to

     uhReadGame, uhWriteGame, uhReadRamrom, uhWriteRamrom, and uhCloseGame,

     presuming the open succeeds.



     The uhReadGame routine will attempt to read nbytes of data from the game

     and copy it to the memory region beginning at addr.  For correct

     operation, it should be paired from the game side with a osWriteHost with

     exactly the same number of bytes.  If the read is attempted from the host

     side before the game has executed the write, the host process will block.

     Conversely, if the write is attempted from the game side before the host

     has executed a read, the game thread will block. When both sides have

     correctly made the corresponding procedure call, the transfer will take

     place and the blocking side will proceed.



     The uhWriteGame routine will attempt to copy nbytes of data from the host

     beginning at addr to the game.  In this case, the operation should be

     paired on the game side with a osReadHost procedure with the same byte

     count.  As above, either the host or game will block until the rendezvous

     is complete.









                                                                        Page 1













uhOpenGame(3P)                                                  uhOpenGame(3P)







     The uhReadRamrom routine will attempt to copy nbytes of data from the

     ramrom memory, begining at ramrom_addr. The calling program must supply a

     buffer, data big enough to hold the requested nbytes. In previous

     releases, calls to uhReadRamrom needed to be paired from the game side

     with a osAckRamromRead. This is no longer neccessary.



     The uhWriteRamrom routine will attempt to write nbytes from data to the

     ramrom memory, begining at ramrom_addr. In previous releases, calls to

     uhWriteRamrom needed to be paired from the game side with a

     osAckRamromWrite. This is no longer neccessary.



     The uhCloseGame procedure can be used if desired to close the given file

     descriptor.



     These routines are part of the Ultra 64 host side library libultrahost.a,

     and thus are available to the application by adding -lultrahost on the

     link line.



EXAMPLE

     The example on the following page shows how a host application can

     initiate the same and establish a connection to transfer data.  This

     particular sequence will guarantee that no handshaking events will be

     lost.  By the time the game sends its first packet of data, the host

     device driver is ready to acknowledge it.  Furthermore, the host side

     application does not attempt to send data to the game before it has

     booted.



























































                                                                        Page 2













uhOpenGame(3P)                                                  uhOpenGame(3P)







        main(int argc, char **argv)     /* Host side code */

        {

            pid_t       pid;

            int         fd, status;



            if ((fd = uhOpenGame("/dev/u64_data")) == -1) {

                perror("uhOpenGame");

                return(-1);

            }

            if ((pid = fork()) == -1) {

                perror("fork);

                return(-1);

            } else if (pid == 0) {

                (void)execl("/usr/sbin/gload", "/usr/sbin/gload", 0);

                fprintf(stderr, "host: execl(

                return(-1);

            }



            if (uhReadGame(fd, hostBuffer, 4) == -1) {

                fprintf(stderr, "uhReadGame %s0, sys_errlist[errno]);

                return(1);

            }



            if (uhCloseGame(fd) != 0) {

                perror("uhCloseGame");

                return(-1);

            }

            if (waitpid(pid, &status, WUNTRACED) == -1) {

                perror("waitpid");

                return(-1);

            }

        }





        mainproc(void *arg)     /* Game side code */

        {

            osWriteHost(gameBuffer, 4);



            osExit();

        }

9

DIAGNOSTICS

     uhOpenGame On success, a valid file descriptor is returned.  Otherwise, a

     -1 is returned and errno is set to indicate the error.



     uhReadGame The number of bytes successfully read is returned.



     uhWriteGame On success, uhWriteGame returns the number of bytes actually

     written.  Otherwise, it returns -1 and sets errno to identify the error.





9





                                                                        Page 3













uhOpenGame(3P)                                                  uhOpenGame(3P)







     uhReadRamrom On success, a value of 0 is returned.  Otherwise, a -1 is

     returned and errno is set to indicate the error.



     uhWriteRamrom On success, a value of 0 is returned. Otherwise, a -1 is

     returned and errno is set to indicate the error.



     uhCloseGame On success, a value of 0 is returned. Otherwise, a -1 is

     returned and errno is set to indicate the error.































































































                                                                        Page 4







