





rspasm(1P)                                                          rspasm(1P)







NAME

     rspasm - Ultra 64 signal processor (RSP) assembler.



SYNOPSIS

     rspasm [options] srcfile



DESCRIPTION

     rspasm is a two-pass assembler for the Ultra 64's Reality Signal

     Processor (RSP).  It translates RSP source code into an RSP executable

     which can be executed on the RSP simulator (rsp), or the RCP hardware.



     The assembly language for the RSP does not conform to the MIPS Assembly

     Language Programming Guide, although in many places it is similar. The

     language supports labels, data symbols, named registers, and simple

     expressions.



     By default, the RSP assembler input passes through '/usr/bin/cc -E' (the

     MIPS ANSI equivalent of the traditional /usr/lib/cpp) so C-preprocessor

     macros such as #define and #include are supported, as well as C comment

     syntax /**/.



     If requested, the output of the C pre-processor is passed through the m4

     macro pre-processor [see m4(1) ], before assembly.



     Similar to other MIPS compilers and assemblers, rspasm defines the name

     _LANGUAGE_ASSEMBLY which you can use around any code that might be shared

     between C and assembly, such as header files.



     There is no linker; a single text segment is built with each compilation.

     There can be only one base address for the text segment.



     A single initialized data segment is permitted, and pseudo-opcodes

     (directives) are provided to control where it is located and what it

     contains.



     Command line options are:



          -D<name>    Define symbol <name> to the C pre-processor.



          -E          Run only the pre-processor(s), don't assemble.



          -I<dir>     Add a directory to the include file search path.



          -S <file>   Object file to search for external symbols.



          -c          Do not run the source through /usr/bin/cc -E (MIPS ANSI

                      C pre-processor)



          -d          Turn on yacc debugging. May be useful for advanced

                      debugging.











                                                                        Page 1













rspasm(1P)                                                          rspasm(1P)







          -h          List command line options.



          -l          Do not generate a listing file.



          -m          Run the source through the m4 macro pre-processor after

                      /usr/bin/cc -E, before assembly.



          -o <file>   Name of output file (default is a.out).



          srcfile is the name of the RSP assembler source code file.



   RSP Instruction Set

     The RSP instruction set is essentially a 32-bit subset of the R4000

     instruction set, with some extensions. Instructions which are not

     implemented include: any 64-bit instruction, mulitplies, divides, branch

     likely, and most system control opcodes.



     The RSP vector unit (VU) is implemented as a MIPS co-processor (COP2),

     with the machine language conforming to the MIPS co-processor definition.

     The RSP assembler uses a mnemonic syntax for each VU instruction.



   RSP Assembler Directives

     The RSP assembler syntax includes pseudo-opcodes which direct

     compilation. Briefly, they are:



          .align <iexpression>

                      The current location of the text or data section is

                      aligned to the next multiple byte boundary corresponding

                      to the evaluated <iexpression>; possibly adding padding.



          .bound <iexpression>

                      This directive performs a check, printing out an error

                      message and aborting compilation if the current location

                      within the text or data section is not aligned to the

                      next multiple byte boundary corresponding to the

                      evaluated <iexpression>.



          .byte <iexpression>

                      Initialize the next byte of the data segment to the

                      value <data>.



          .data [<expression>]

                      Switch to the data section. All data initialization

                      directives must be contained in the data section.



                      If the optional <expression> is present, it is evaluated

                      and used as the base address to continue packing the

                      data section. Only the least significant 12 bits of the

                      base addres is used, since DMEM is only 4K bytes.



                      Multiple base addresses are permitted, any "holes"

                      between initialized data will remain un-initialized. The







                                                                        Page 2













rspasm(1P)                                                          rspasm(1P)







                      assembler keeps track of the maximum address

                      initialized, and all data up to that point (including

                      any holes) will be output.



          .dmax <iexpression>

                      This directive performs a check, printing out an error

                      message and aborting compilation if the current location

                      within the text or data section exceeds the value

                      corresponding to the evaluated <iexpression>.



          .end <identifier> [, <expression>]

                      End a procedure. The assembler outputs debugging

                      information for the debugger, including the beginning

                      and ending locations of procedures.



          .ent <identifier> [, <expression>]

                      Begin a procedure. The assembler outputs debugging

                      information for the debugger, including the beginning

                      and ending locations of procedures.



          .half <identifer> | <iexpression>

                      Initialize the next half-word (16 bits) of the data

                      segment to the value <data>. Aligns to 16-bit boundary.



          .name, <identifier>, <register>

                      Associate the name <identifier> with the register

                      <register>.



          .print <string> [, <expression>][, <expression>] ...

                      The quoted <string> constant is printed to stderr during

                      program assembly.



                      The string constant may contain C-like numeric printf

                      conversions (%d, %x, etc.) and the <expression>'s will

                      be evaluated and printed to stderr.



                      A maximum of four <expressions>'s are permitted per

                      .print directive.



          .space <expression>

                      If we are in the data section, <expression> number of

                      bytes are allocated and filled with zeros. The new

                      current location of the data section will be equal to

                      the previous location, plus <expression> bytes.



                      If we are in the text section, <expression >> 2 number

                      of instructions are padded and filled with nop's and the

                      new program counter for assembly will be equal to the

                      old program counter, plus <expression> bytes.













                                                                        Page 3













rspasm(1P)                                                          rspasm(1P)







          .symbol <identifier>, <expression>

                      The <identifier> is entered into the symbol table with

                      the value of <expression>.



          .text [<expression>]

                      Switch to the text section. All program instructions

                      must be contained in the text section.



                      If the optional <expression> is present, it is evaluated

                      and used as the base address for assembling the program.

                      Only the least significant 12 bits of the base address

                      is used, since IMEM is only 4K bytes.



                      Only one .text directive with a base address may be

                      present, otherwise unpredictable program assembly will

                      result.



          .unname <identifier>

                      Removes <identifier> from the symbol table.



          .word <identifier> | <iexpression>

                      Initialize the next word (32-bits) to the value of the

                      <identifer> or <iexpression>.





   Assembler Output

     The rspasm assembler outputs several specially-formatted files. The root

     filename for these files can be specified with the -o flag.



          <rootname>  is the binary executable code (text section). This file

                      can be loaded into the RSP simulator and executed.



          <rootname>.dat

                      is the binary data section. This file is usually loaded

                      into DMEM.



          <rootname>.lst

                      is a text program listing.



          <rootname>.sym

                      is a "symbol file" used by the RSP simulator to perform

                      source level debugging



          <rootname>.dbg

                      is a "symbol file" used by the rsp2elf utility in order

                      to build an ELF object that can be used with makerom and

                      the gvd debugger.





   BNF Specification of the RSP Assembly Language

     This section presents a formal specification of the RSP assembly language

     using a Backus-Naur Form (BNF).







                                                                        Page 4













rspasm(1P)                                                          rspasm(1P)







     <program> ::= <instruction> | <program> <instruction>





     <instruction> ::= <directive> |

          <label> <directive> |

          <label> <label> <directive> |

          <scalarInstruction> |

          <label> <scalarInstruction> |

          <vectorInstruction> |

          <label> <vectorInstruction>





     <directive> ::= .align <iexpression> |

          .bound <iexpression> |

          .byte <iexpression> |

          .data |

          .data <iexpression>

          .dmax <iexpression>

          .end |

          .end <identifier> |

          .ent <identifier> |

          .ent <identifier> , <integer> |

          .half <identifier> |

          .half <iexpression> |

          .name <identifier> , <scalarRegister> |

          .name <identifier> , <vectorRegister> |

          .name <identifier> , <controlRegister> |

          .print <qstring> |

          .print <qstring> , <expression> |

          .print <qstring> , <expression> , <expression> |

          .print <qstring> , <expression> , <expression> ,

               <expression> |

          .print <qstring> , <expression> , <expression> ,

               <expression> , <expresion> |

          .space <expression> |

          .symbol <identifier> , <expression> |

          .text |

          .text <expression> |

          .unname <identifier> |

          .word <identifier> |

          .word <iexpression>





     <scalarInstruction> ::= <regOp> <scalarRegister> |

          <regRegOp> <scalarRegister> |

          <regRegOp> <scalarRegister> , <scalarRegister> |

          <regRegOp> <scalarRegister> , <controlRegister> |

          <regRegRegOp> <scalarRegister> , <scalarRegister> ,

               <scalarRegister> |

          <regImmOp> <scalarRegister> , <expression> |

          <regRegImmOp> <scalarRegister> , <expression> |

          <regRegImmOp> <scalarRegister> , <scalarRegister> ,







                                                                        Page 5













rspasm(1P)                                                          rspasm(1P)







          <expression> |

          <regOffsetOp> <scalarRegister> , <expression> |

          <regOffsetOp> <expression> |

          <regRegOffsetOp> <scalarRegister> , <scalarRegister> ,

               <expression> |

          <regOffsetBaseOp> <scalarRegister> , <expression> (

               <scalarRegister> ) |

          <regRegShiftOp> <scalarRegister> , <scalarRegister> ,

               <expression> |

          <sRegRegRegOp> <scalarRegister> , <scalarRegister> ,

               <scalarRegister> |

          <targetOp> <expression> |

          <vRegsRegOp> <vectorRegister> [ <element> ] ,

               <expression> ( <scalarRegister> ) |

          <sRegvRegOp> <scalarRegister> , <vectorRegister> |

          <sRegvRegOp> <scalarRegister> , <vectorRegister> [ <element> ] |

          <noOperandOp>





     <vectorInstruction> ::= <veRegvRegvRegOp> <vectorRegister> ,

               <vectorRegister> , <vectorRegister> |

          <veRegvRegvRegOp> <vectorRegister> ,

               <vectorRegister> , <vectorRegister> [ <element> ] |

          <vdRegvRegOp> <vectorRegister> [ <element> ] ,

               <vectorRegister> [ <element> ]





     <regOp> ::= jr





     <regRegRegOp> ::= add | addu | and | nor | or | slt | sltu

               | sub | subu | xor





     <regImmOp> ::= lui





     <regRegImmOp> ::= addi | addiu | andi | ori | slti | sltiu | xori





     <regOffsetOp> ::= bgez | bgezal | bgtz | blez | bltz | bltzal





     <regRegOffsetOp> ::= beq | bne





     <regOffsetBaseOp> ::= lb | lbu | lw | lh | lhu | sb | sh | sw





     <regRegShiftOp> ::= sll | sra | srl











                                                                        Page 6













rspasm(1P)                                                          rspasm(1P)







     <sregRegRegOp> ::= sllv | srav | srlv





     <targetOp> ::= j | jal





     <vRegsRegOp> ::= lbv | lsv | llv | ldv | lqv | lrv | lpv |

               luv | lhv | lfv | ltv | sbv | ssv | slv | sdv |

               sqv | srv | spv | suv | shv | sfv | swv | stv





     <sRegvRegOp> ::= mfc2 | cfc2 | mtc2 | ctc2





     <noOperandOp> ::= nop | vnop | break





     <veRegvRegvRegOp> ::= vmulf | vmacf | vmulu | vmacu |

               vrndp | vrndn | vmulq | vmacq | vmudh | vmadh | vmudm |

               vmadm | vmudn | vmadn | vmudl | vmadl | vadd | vsub |

               vabs | vaddc | vsubc | vsar | vand | vnand | vor | vnor |

               vxor | vnxor | vlt | veq | vne | vge | vcl | vch |

               vcr | vmrg





     <vdRegvRegOp> ::= vmov | vrcp | vrsq | vrcph | vrsqh | vrcpl | vrsql





     <expression> ::= ( <expression> ) |

          <integer> |

          <identifier> |

          ~ <expression> |

          <expression> & <expression> |

          <expression> | <expression> |

          <expression> ^ <expression> |

          <expression> << <expression> |

          <expression> >> <expression> |

          <expression> * <expression> |

          <expression> / <expression> |

          <expression> % <expression> |

          <expression> + <expression> |

          <expression> - <expression> |

          - <expression> |

          + <expression>





     <iexpression> ::= ( <iexpression> ) |

          <integer> |

          ~ <iexpression> |

          <iexpression> & <iexpression> |

          <iexpression> | <iexpression> |

          <iexpression> ^ <iexpression> |







                                                                        Page 7













rspasm(1P)                                                          rspasm(1P)







         <iexpression> << <iexpression> |

          <iexpression> >> <iexpression> |

          <iexpression> * <iexpression> |

          <iexpression> / <iexpression> |

          <iexpression> % <iexpression> |

          <iexpression> + <iexpression> |

          <iexpression> - <iexpression> |

          - <iexpression> |

          + <iexpression>





     <scalarRegister> ::= <identifier> |$<integer> | $sp | $s8 | $at | $ra





     <vectorRegister> ::= <identifier> | $v<integer> | $vco | $vcc | $vce





     <controlRegister> ::= <identifier> | $c<integer>





     <element> ::= <iexpression> | <iexpression>h | <iexpression>q





     <identifier> ::= <alpha> <alphanumeric>*





     <alphanumeric> ::= {<alpha> | <digit> | _}*





     <qstring> ::= " {<ASCII text> | <whitespace> |

               <C print specifier>}* "





     <alpha> ::= a | b | c | d | e | f | g | h | i | j | k | l |

               m | n | o | p | r | s | t | u | v | w | x | y | z |

               A | B | C | D | E | F | G | H | I | J | K | L | M |

               N | O | P | Q | R | S | T | U | V | W | X | Y | Z





     <integer> ::= <digit>* | 0x<hexdigit>* | 0X<hexdigit>* |

               0<octdigit>*





     <digit> ::= 0 | 1 | 2 | 3 | 4 | 5 | 6 | 7 | 8 | 9





     <hexdigit> ::= <digit> | a | b | c | d | e | f |

               A | B | C | D | E | F





     <octdigit> ::= 0 | 1 | 2 | 3 | 4 | 5 | 6 | 7









                                                                        Page 8













rspasm(1P)                                                          rspasm(1P)







NOTE

     The RSP Assembly Language is a proprietary interface.





SEE ALSO

     rsp(1P), m4(1), Nintedo Ultra64 RSP Programmer's Guide



































































































                                                                        Page 9







